package cn.ibizlab.core.data.service;

import cn.ibizlab.core.data.domain.DOModel;
import cn.ibizlab.core.data.dto.BaseData;
import cn.ibizlab.core.data.dto.FilterData;
import cn.ibizlab.core.data.dto.RequestData;
import cn.ibizlab.core.data.dto.ResponseData;
import cn.ibizlab.core.data.model.DSLink;
import cn.ibizlab.core.data.model.POSchema;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;

import java.io.Serializable;
import java.util.List;

public interface IPersistentService {


    BaseData create(DSLink link, POSchema poSchema, BaseData et, boolean bGet);
    List<BaseData> createBatch(DSLink link,POSchema poSchema,List<BaseData> list,boolean bGet);

    BaseData update(DSLink link,POSchema poSchema,BaseData et,boolean bGet);
    List<BaseData> updateBatch(DSLink link,POSchema poSchema,List<BaseData> list,boolean bGet);

    boolean remove(DSLink link,POSchema poSchema,Serializable key);
    boolean removeBatch(DSLink link,POSchema poSchema,List<Serializable> idList);
    boolean removeByMap(DSLink link,POSchema poSchema,BaseData et);
    boolean removeByMapBatch(DSLink link,POSchema poSchema,List<BaseData> list);

    BaseData get(DSLink link,POSchema poSchema,Serializable key);
    List<BaseData> getBatch(DSLink link,POSchema poSchema,List<Serializable> idList);
    BaseData getByMap(DSLink link,POSchema poSchema,BaseData et);
    List<BaseData> getByMapBatch(DSLink link,POSchema poSchema,List<BaseData> list);

    List<BaseData> getBatchKey(DSLink link,POSchema poSchema,List<BaseData> list);

    Long count(DSLink link,POSchema poSchema,BaseData et);
    boolean checkKey(DSLink link,POSchema poSchema,BaseData et);

    BaseData save(DSLink link,POSchema poSchema,BaseData et,boolean bGet);
    List<BaseData> saveBatch(DSLink link,POSchema poSchema,List<BaseData> list,boolean bGet);

    List<BaseData> select(DSLink link,POSchema poSchema,BaseData et);
    Page<BaseData> select(DSLink link,POSchema poSchema,BaseData et,Pageable pageable);

    List<BaseData> query(DSLink link,POSchema poSchema,String dataQuery,FilterData context);
    Page<BaseData> query(DSLink link,POSchema poSchema,String dataQuery,FilterData context,Pageable pageable);

    List<BaseData> selectRaw(DSLink link,POSchema poSchema,String sql,FilterData context);
    boolean execRaw(DSLink link,POSchema poSchema,String sql,BaseData param);

    List<BaseData> fetch(DSLink link,POSchema poSchema,String dataSet,FilterData context);
    Page<BaseData> fetch(DSLink link,POSchema poSchema,String dataSet,FilterData context,Pageable pageable);


 
 
    default BaseData create(String datasource,String table,BaseData et,boolean bGet)
    {
        DSLink link=getDSLink(datasource);
        return create(link,getSchema(link,table),et,bGet);
    }
    default  List<BaseData> createBatch(String datasource,String table,List<BaseData> list,boolean bGet)
    {
        DSLink link=getDSLink(datasource);
        return createBatch(link,getSchema(link,table),list,bGet);
    }
    default BaseData update(String datasource,String table,BaseData et,boolean bGet)
    {
        DSLink link=getDSLink(datasource);
        return update(link,getSchema(link,table),et,bGet);
    }
    default List<BaseData> updateBatch(String datasource,String table,List<BaseData> list,boolean bGet)
    {
        DSLink link=getDSLink(datasource);
        return updateBatch(link,getSchema(link,table),list,bGet);
    }
    default boolean remove(String datasource,String table,Serializable key)
    {
        DSLink link=getDSLink(datasource);
        return remove(link,getSchema(link,table),key);
    }
    default boolean removeBatch(String datasource,String table,List<Serializable> idList)
    {
        DSLink link=getDSLink(datasource);
        return removeBatch(link,getSchema(link,table),idList);
    }
    default boolean removeByMap(String datasource,String table,BaseData et)
    {
        DSLink link=getDSLink(datasource);
        return removeByMap(link,getSchema(link,table),et);
    }
    default boolean removeByMapBatch(String datasource,String table,List<BaseData> list)
    {
        DSLink link=getDSLink(datasource);
        return removeByMapBatch(link,getSchema(link,table),list);
    }
    default BaseData get(String datasource,String table,Serializable key)
    {
        DSLink link=getDSLink(datasource);
        return get(link,getSchema(link,table),key);
    }
    default List<BaseData> getBatch(String datasource,String table,List<Serializable> idList)
    {
        DSLink link=getDSLink(datasource);
        return getBatch(link,getSchema(link,table),idList);
    }
    default BaseData getByMap(String datasource,String table,BaseData et)
    {
        DSLink link=getDSLink(datasource);
        return getByMap(link,getSchema(link,table),et);
    }
    default List<BaseData> getByMapBatch(String datasource,String table,List<BaseData> list)
    {
        DSLink link=getDSLink(datasource);
        return getByMapBatch(link,getSchema(link,table),list);
    }
    default List<BaseData> getBatchKey(String datasource,String table,List<BaseData> list)
    {
        DSLink link=getDSLink(datasource);
        return getBatchKey(link,getSchema(link,table),list);
    }
    default Long count(String datasource,String table,BaseData et)
    {
        DSLink link=getDSLink(datasource);
        return count(link,getSchema(link,table),et);
    }
    default boolean checkKey(String datasource,String table,BaseData et)
    {
        DSLink link=getDSLink(datasource);
        return checkKey(link,getSchema(link,table),et);
    }
    default BaseData save(String datasource,String table,BaseData et,boolean bGet)
    {
        DSLink link=getDSLink(datasource);
        return save(link,getSchema(link,table),et,bGet);
    }
    default List<BaseData> saveBatch(String datasource,String table,List<BaseData> list,boolean bGet)
    {
        DSLink link=getDSLink(datasource);
        return saveBatch(link,getSchema(link,table),list,bGet);
    }
    default List<BaseData> select(String datasource,String table,BaseData et){
        DSLink link=getDSLink(datasource);
        return select(link,getSchema(link,table),et);
    }
    default Page<BaseData> select(String datasource,String table,BaseData et,Pageable pageable){
        DSLink link=getDSLink(datasource);
        return select(link,getSchema(link,table),et,pageable);
    }
    default List<BaseData> query(String datasource,String table,String dataQuery,FilterData context){
        DSLink link=getDSLink(datasource);
        return query(link,getSchema(link,table),dataQuery,context);
    }
    default Page<BaseData> query(String datasource,String table,String dataQuery,FilterData context,Pageable pageable){
        DSLink link=getDSLink(datasource);
        return query(link,getSchema(link,table),dataQuery,context,pageable);
    }
    default List<BaseData> selectRaw(String datasource,String table,String sql,FilterData context){
        DSLink link=getDSLink(datasource);
        return selectRaw(link,getSchema(link,table),sql,context);
    }
    default boolean execRaw(String datasource,String table,String sql,BaseData param){
        DSLink link=getDSLink(datasource);
        return execRaw(link,getSchema(link,table),sql,param);
    }
    default List<BaseData> fetch(String datasource,String table,String dataSet,FilterData context){
        DSLink link=getDSLink(datasource);
        return fetch(link,getSchema(link,table),dataSet,context);
    }
    default Page<BaseData> fetch(String datasource,String table,String dataSet,FilterData context,Pageable pageable){
        DSLink link=getDSLink(datasource);
        return fetch(link,getSchema(link,table),dataSet,context,pageable);
    }
    
    DSLink getDSLink(String datasource);

    POSchema getSchema(DSLink link,String table);

}
